<?php
namespace PME\Modules\Planejamento\Cadastro\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class CentralController extends \PME\Services\TeknisaCrud {

	protected $crudService;
	protected $parameterBag;
	protected $entityManager;
	protected $connection;
	protected $environment;

    public function __construct($crudService, $parameterBag, $entityManager, $connection, $environment) {
    	$this->entityManager = $entityManager;
    	$this->connection = $connection;
    	$this->environment = $environment;
        parent::__construct($crudService, $parameterBag);
	}

	public function excluiCentral(DTO\Request $request, DTO\Response $response) {
		try {		
			$params = $request->getParameters()["checkedRows"];
            $this->connection->beginTransaction();
			
			foreach ($params as $param) {
				$validacao = $this->validaCentral($param);
				
				if ($validacao !== true) {
					$this->connection->rollBack();
					$response->addDataset(new DataSet('mensagem', array($validacao)));
					return;
				};
				
				$sql = "UPDATE FILIAL 
				           SET IDTPFILIALCENT = NULL, IDFILCENTPME = 'N',
				               CDFILCENTPME = NULL
                         WHERE CDFILIAL = :CDFILIAL 
                           AND NRORG = :NRORG";
                
                $params = array(
                    'CDFILIAL' => $param['CDFILIAL'],
                    'NRORG' => $this->environment->getNrOrgTrab()
                );
                
                $this->entityManager->getConnection()->executeUpdate($sql, $params);
			}
			
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function salvaCentral(DTO\Request $request, DTO\Response $response) {
		try {			
			$params = $request->getParameters()["editedRows"];
            $this->connection->beginTransaction();
			
			foreach ($params as $param) {
				if ($param['__is_new'] === true) {
					$validacao = $this->validaUnidade($param);
					
					if ($validacao !== true) {
						$this->connection->rollBack();
						$response->addDataset(new DataSet('mensagem', array($validacao)));
						return;
					};
				}
			
				// Atualiza unidade como central conforme o tipo passado.
				$sql = "UPDATE FILIAL 
				           SET IDTPFILIALCENT = :IDTPFILIALCENT, IDFILCENTPME = 'S' 
                         WHERE CDFILIAL = :CDFILIAL 
                           AND NRORG = :NRORG";
                
                $paramsUp = array(
                    'CDFILIAL' => $param['CDFILIAL'],
                    'NRORG' => $this->environment->getNrOrgTrab(),
                    'IDTPFILIALCENT' => $param['IDTPFILIALCENT']
                );
                
                $this->entityManager->getConnection()->executeUpdate($sql, $paramsUp);
			}
			
            $this->connection->commit();
        }catch (\Exception $e) {
            $this->connection->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function salvaAgregada(DTO\Request $request, DTO\Response $response) {
		try {			
			$params = $request->getParameters()["editedRows"];
            $this->connection->beginTransaction();
			
			foreach ($params as $param) {
				$validacao = $this->validaAgregada($param);
				
				if ($validacao !== true) {
					$this->connection->rollBack();
					$response->addDataset(new DataSet('mensagem', array($validacao)));
					return;
				};
				
				$sql = "UPDATE FILIAL 
				           SET CDFILCENTPME = :CDFILCENTPME
                         WHERE CDFILIAL = :CDFILIAL 
                           AND NRORG = :NRORG";
                
                $paramsUp = array(
                    'CDFILIAL' => $param['CDFILIAL'],
                    'NRORG' => $this->environment->getNrOrgTrab(),
                    'CDFILCENTPME' => $param['CDFILCENTPME']
                );
                
                $this->entityManager->getConnection()->executeUpdate($sql, $paramsUp);
			}
			
            $this->connection->commit();
        }catch (\Exception $e) {
            $this->connection->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	private function validaUnidade($param){
		// Valida se unidade já é agregada de outra
		$sql = "SELECT A.CDFILIAL, A.CDFILCENTPME, C.NMFILIAL
				  FROM FILIAL A
                  INNER JOIN FILIAL C ON C.CDFILIAL = A.CDFILCENTPME 
                					 AND C.NRORG = A.NRORG 
                 WHERE A.CDFILIAL = :CDFILIAL 
                   AND A.NRORG = :NRORG";
        
        $paramSql = array(
            'CDFILIAL' => $param['CDFILIAL'],
            'NRORG' => $this->environment->getNrOrgTrab()
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_E_AGREGADA:' . $resultado[0]['CDFILCENTPME'] . ' - ' . $resultado[0]['NMFILIAL'];
        }
        
        // Valida se unidade já é cadastrada como central
		$sql = "SELECT CDFILIAL, NMFILIAL
				  FROM FILIAL 
                 WHERE CDFILIAL = :CDFILIAL 
                   AND NRORG = :NRORG
                   AND IDFILCENTPME = 'S'";
        
        $paramSql = array(
            'CDFILIAL' => $param['CDFILIAL'],
            'NRORG' => $this->environment->getNrOrgTrab()
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_JA_CADASTRADA:' . $resultado[0]['CDFILIAL'] . ' - ' . $resultado[0]['NMFILIAL'];
        }
        
        return true;
	}
	
	private function validaCentral($param){
		// Valida se central tem agregadas além dela
		$sql = "SELECT A.CDFILIAL, A.CDFILCENTPME, A.NMFILIAL
				  FROM FILIAL A
                 WHERE A.CDFILCENTPME = :CDFILIAL 
                   AND A.CDFILIAL <> A.CDFILCENTPME
                   AND A.NRORG = :NRORG";
        
        $paramSql = array(
            'CDFILIAL' => $param['CDFILIAL'],
            'NRORG' => $this->environment->getNrOrgTrab()
        );
        
        $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
        if (count($resultado) > 0) {
        	return 'ERROR_TEM_AGREGADA';
        }
        
        return true;
	}
	
	private function validaAgregada($param){
	    if ($param['CDFILIAL'] <> $param['CDFILCENTPME']) { // Valida se unidade já é cadastrada como central e não é a atual
    // 		$sql = "SELECT CDFILIAL, NMFILIAL
    // 				  FROM FILIAL 
    //                  WHERE CDFILCENTPME = :CDFILIAL 
    //                   AND NRORG = :NRORG";
            
    //         $paramSql = array(
    //             'CDFILIAL' => $param['CDFILIAL'],
    //             'NRORG' => $this->environment->getNrOrgTrab()
    //         );
            
            $sql2 = "SELECT
                        CDFILIAL
                     FROM 
                        FILIAL
                     WHERE 
                        IDFILCENTPME = :IDFILCENTPME
                        AND NRORG = :NRORG";
                        
            $paramSql2 = array(
                'IDFILCENTPME' => 'S',
                'NRORG' => $this->environment->getNrOrgTrab()
            );
            
            // $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
            
            $resultado2 = $this->entityManager->getConnection()->fetchAll($sql2, $paramSql2);
            
            foreach ($resultado2 as $result) {
                if (strcasecmp($result['CDFILIAL'], $param['CDFILIAL']) == 0){
                    return 'ERROR_UNID_CENTRAL';
                }
            }
            
            // if (count($resultado) > 0) {
            // 	return 'ERROR_UNID_CENTRAL';
            // }
	    }
        
        return true;
	}
	
	public function excluiAgregada(DTO\Request $request, DTO\Response $response) {
		try {		
			$params = $request->getParameters()["checkedRows"];
            $this->connection->beginTransaction();
			
			foreach ($params as $param) {
				$sql = "UPDATE FILIAL 
				           SET CDFILCENTPME = NULL 
                         WHERE CDFILIAL = :CDFILIAL 
                           AND NRORG = :NRORG";
                
                $params = array(
                    'CDFILIAL' => $param['CDFILIAL'],
                    'NRORG' => $this->environment->getNrOrgTrab()
                );
                
                $this->entityManager->getConnection()->executeUpdate($sql, $params);
			}
			
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
}