import {
  Loader,
  Metadata,
  IDictionary,
  Http,
} from '@zeedhi/core';

import {
  GridEditable, Form, DialogService,
} from '@zeedhi/common';
  
export class Ger30002PadraoDeServicoController {
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public modalController = Loader.getInstance('ModalController');

  public crudController = Loader.getInstance('CrudController');

  public utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  public confirmaAcaoAtualizacao() {
    const gridPadraoServico: GridEditable = Metadata.getInstance('gridPadraoDeServico');
    const numLinhasSelecionadasGrid = gridPadraoServico.selectedRows.length;
    DialogService.show({
      name: 'confirmarAcaoAtualizacaoSimOuNao',
      type: 'warning',
      title: 'pme::atencao',
      text: (numLinhasSelecionadasGrid > 1) ? 'pme::desejaAtualizarServicosPlural' : 'pme::desejaAtualizarServicosSingular',
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'pme::sim',
          events: { click: () => this.realizaAcaoAtualizacao(gridPadraoServico) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'pme::nao',
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }
  
  private realizaAcaoAtualizacao(gridPadraoServico: GridEditable) {
    const padroesServicosSelecionados = gridPadraoServico.selectedRows;
    const formModalPadraoServico: Form = Metadata.getInstance('formModalPadraoServico');
    let unidadesSelecionadasForm: IDictionary = [];
    if (formModalPadraoServico.value.UNIDADE.length > 0) {
      unidadesSelecionadasForm = formModalPadraoServico.value.UNIDADE;
    } else {
      unidadesSelecionadasForm = [];
    }
    Http.post('/updateAtualizacaoAutomatica', { padroesServicosSelecionados, unidadesSelecionadasForm }).then(() => {
      this.utilities.showAlert('alert', 'pme::atualizadoComSucesso', 'success');
      DialogService.hide();
      this.modalController.closeModal();
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
      DialogService.hide();
      this.modalController.closeModal();
    });
  }
}
